//
//  igViewController.m
//  FormatStringSpeedTest
//
//  Created by Torrey Betts on 10/21/13.
//  Copyright (c) 2013 Infragistics. All rights reserved.
//

#import <IGChart/IGChart.h>
#import "igViewController.h"

@interface ComparisonItem : NSObject
@property (nonatomic, assign) int formatNSString;
@property (nonatomic, assign) int formatSprintf;
@end

@implementation ComparisonItem
@end

@interface igViewController () <IGChartViewDelegate>
{
    UILabel *_label;
    UIButton *_button;
    IGChartView *_chart;
    IGLegend *_legend;
    NSMutableArray *_data;
    IGCategorySeriesDataSourceHelper *_dshNSString;
    IGCategorySeriesDataSourceHelper *_dshSprintf;
    IGColumnSeries *formatNSStringSeries, *formatSprintfSeries;
}
@end

@implementation igViewController

- (void)viewDidLoad
{
    [super viewDidLoad];

    self.view.backgroundColor = [UIColor whiteColor];
    _data = [[NSMutableArray alloc] init];
    CGFloat splitHeight = (self.view.bounds.size.height - 50) / 2;
    CGFloat chartWidth = self.view.bounds.size.width * 0.8f;
    CGFloat legendWidth = self.view.bounds.size.width - chartWidth;

    _label = [[UILabel alloc] init];
    _label.frame = CGRectInset(CGRectMake(0, 50 + splitHeight, self.view.bounds.size.width, splitHeight), 10, 10);
    _label.autoresizingMask = UIViewAutoresizingFlexibleHeight|UIViewAutoresizingFlexibleWidth|UIViewAutoresizingFlexibleTopMargin;
    _label.numberOfLines = 0;
    _label.textAlignment = NSTextAlignmentCenter;
    [self.view addSubview:_label];

    _button = [UIButton buttonWithType:UIButtonTypeRoundedRect];
    _button.frame = CGRectMake(5, 0, 200, 50);
    _button.autoresizingMask = UIViewAutoresizingFlexibleWidth|UIViewAutoresizingFlexibleHeight|UIViewAutoresizingFlexibleBottomMargin|UIViewAutoresizingFlexibleRightMargin;
    [_button setTitle:@"Run Test" forState:UIControlStateNormal];
    [_button addTarget:self action:@selector(buttonTapped) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:_button];

    _chart = [[IGChartView alloc] init];
    _chart.frame = CGRectMake (0, 50, chartWidth, splitHeight);
    _chart.autoresizingMask = UIViewAutoresizingFlexibleWidth|UIViewAutoresizingFlexibleHeight|UIViewAutoresizingFlexibleTopMargin|UIViewAutoresizingFlexibleRightMargin|UIViewAutoresizingFlexibleBottomMargin;
    _chart.isSquare = YES;
    _chart.delegate = self;
    _chart.theme = [IGChartGradientThemes IGTheme];
    [self.view addSubview:_chart];

    _legend = [[IGLegend alloc] initWithLegendType:IGChartLegendTypeSeries];
    _legend.frame = CGRectMake (chartWidth, 50, legendWidth - 5, splitHeight);
    _legend.autoresizingMask = UIViewAutoresizingFlexibleWidth|UIViewAutoresizingFlexibleHeight|UIViewAutoresizingFlexibleTopMargin|UIViewAutoresizingFlexibleLeftMargin|UIViewAutoresizingFlexibleBottomMargin;
    _legend.orientation = IGOrientationVertical;
    _legend.horizontalAlignment = IGHorizontalAlignCenter;
    _legend.verticalAlignment = IGVerticalAlignCenter;
    _legend.layer.borderColor = [UIColor grayColor].CGColor;
    _legend.layer.borderWidth = 2;
    _chart.legend = _legend;
    [self.view addSubview:_legend];

    [self performSelector:@selector(executeTest) withObject:nil afterDelay:0.25];
}

- (NSString *)chartView:(IGChartView *)chartView labelForAxis:(IGAxis *)axis withItem:(NSObject *)item
{
    if ([axis.key isEqualToString:@"xAxis"])
    {
        return [((IGCategoryPoint *) item).label stringByReplacingOccurrencesOfString:@"Item" withString:@"Run"];
    }
    else if ([axis.key isEqualToString:@"yAxis"])
    {
        return [NSString stringWithFormat:@"%d ms", [((NSNumber *)item) intValue]];
    }

    return @"";
}

-(void)buttonTapped
{
    [self runTest];
    [self addPoint];
}

-(void)executeTest
{
    [self runTest];

    _dshNSString = [[IGCategorySeriesDataSourceHelper alloc] initWithData:_data andValuePath:@"formatNSString"];
    _dshSprintf = [[IGCategorySeriesDataSourceHelper alloc] initWithData:_data andValuePath:@"formatSprintf"];

    formatNSStringSeries = (IGColumnSeries *)[_chart addSeriesForType:[IGColumnSeries class] usingKey:@"formatNSStringSeries" withDataSource:_dshNSString firstAxisKey:@"xAxis" secondAxisKey:@"yAxis"];
    formatNSStringSeries.title = @"NSString Format";
    formatNSStringSeries.transitionDuration = 0.5;
    formatNSStringSeries.yAxis.minimum = 0;
    formatSprintfSeries = (IGColumnSeries *)[_chart addSeriesForType:[IGColumnSeries class] usingKey:@"formatSprintfSeries" withDataSource:_dshSprintf firstAxisKey:@"xAxis" secondAxisKey:@"yAxis"];
    formatSprintfSeries.title = @"sprintf Format";
    formatSprintfSeries.transitionDuration = 0.5;
}

-(void)addPoint
{
    [_chart insertItemAtIndex:_data.count - 1 withSource:_dshNSString];
    [_chart insertItemAtIndex:_data.count - 1 withSource:_dshSprintf];
}

-(void)runTest
{
    ComparisonItem *comparisonItem = [[ComparisonItem alloc] init];

    //Test NSString stringWithFormat:
    NSDate *start = [NSDate date];
    for (int i = 0; i < 100000; i++) {
        NSString *s = [NSString stringWithFormat:@"Hi %@; Hi to you %@.", @"Torrey", @"too"];
    }
    NSDate *end = [NSDate date];
    NSString *span = [NSString stringWithFormat:@"%.0f", ([end timeIntervalSinceDate:start] * 1000)];
    comparisonItem.formatNSString = (int)([end timeIntervalSinceDate:start] * 1000);
    _label.text = [NSString stringWithFormat:@"Format = %.0f milliseconds", ([end timeIntervalSinceDate:start] * 1000)];

    //Test sprintf that is converted to NSString
    start = [NSDate date];
    for (int i=0; i < 100000; i++) {
        char buffer[64];
        sprintf(buffer, "Hi %s; Hi to you %s.", "Torrey", "too");
        NSString *s = [NSString stringWithCString:buffer encoding:NSUTF8StringEncoding];
    }
    end = [NSDate date];
    span = [NSString stringWithFormat:@"%.0f", ([end timeIntervalSinceDate:start] * 1000)];
    comparisonItem.formatSprintf = (int)([end timeIntervalSinceDate:start] * 1000);
    _label.Text = [NSString stringWithFormat:@"%@\nsprintf = %.0f millseconds", _label.text, ([end timeIntervalSinceDate:start] * 1000)];

    [_data addObject:comparisonItem];
}

- (BOOL)prefersStatusBarHidden
{
    return YES;
}

@end